Imports System.Drawing.Imaging

Public Class Form1
    Private Const Wxmin As Double = -2D
    Private Const Wxmax As Double = 1D
    Private Const Wymin As Double = -1.5D
    Private Const Wymax As Double = 1.5D

    Private m_Bitmap As Bitmap = Nothing
    Private m_Drawing As Boolean = False
    Private m_Rand As New Random

    Private Sub mnuFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileSave.Click
        If sfdBrot.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim file_name As String = sfdBrot.FileName
            If Not file_name.Contains(".") Then file_name &= ".bmp"
            Dim ext As String = file_name.Substring(file_name.LastIndexOf(".")).ToLower()

            Select Case ext
                Case ".bmp"
                    m_Bitmap.Save(file_name, ImageFormat.Bmp)
                Case ".bmgif"
                    m_Bitmap.Save(file_name, ImageFormat.Gif)
                Case ".jpg", ".jpeg"
                    m_Bitmap.Save(file_name, ImageFormat.Jpeg)
                Case ".png"
                    m_Bitmap.Save(file_name, ImageFormat.Png)
                Case ".tif", ".tiff"
                    m_Bitmap.Save(file_name, ImageFormat.Tiff)
                Case Else
                    MessageBox.Show("Unknown file type " & ext)
            End Select
        End If
    End Sub

    Private Sub btnDraw_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDraw.Click
        If btnDraw.Text = "Draw" Then
            btnDraw.Text = "Stop"
            DrawBrot()
        End If

        m_Drawing = False
        btnDraw.Text = "Draw"
    End Sub

    ' Draw the Buddhabrot until stopped or
    ' we plot the desired number of points.
    Private Sub DrawBrot()
        ' Get parameters.
        Dim wid As Integer = Val(txtWidth.Text)
        Dim hgt As Integer = Val(txtHeight.Text)
        Dim cut_r As Integer = Val(txtRedCutoff.Text)
        Dim cut_g As Integer = Val(txtGreenCutoff.Text)
        Dim cut_b As Integer = Val(txtBlueCutoff.Text)
        Dim stop_after As Integer = Val(txtStopAfter.Text)
        Dim draw_every As Integer = Val(txtDrawEvery.Text)

        If wid <= 0 OrElse hgt <= 0 Then
            MessageBox.Show("Invalid parameter", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End If

        ' Make hit count arrays.
        Dim hit_r(wid - 1, hgt - 1) As Integer
        Dim hit_g(wid - 1, hgt - 1) As Integer
        Dim hit_b(wid - 1, hgt - 1) As Integer

        ' Make the bitmap.
        m_Bitmap = New Bitmap(wid, hgt)
        picCanvas.Image = m_Bitmap
        Me.ClientSize = New Size( _
            picCanvas.Left + picCanvas.Width + 8, _
            System.Math.Max( _
                btnDraw.Top + btnDraw.Height, _
                picCanvas.Top + picCanvas.Height) + 8)
        mnuFileSave.Enabled = True

        ' Start drawing.
        Dim start_time As Date = Now
        Dim stop_time As Date
        Dim elapsed As TimeSpan
        m_Drawing = True

        ' Build the hit counts.
        Dim dx As Double = (Wxmax - Wxmin) / hgt
        Dim dy As Double = (Wymax - Wymin) / wid
        Dim max_r, max_g, max_b, hits, total_hits As Integer

        Do While total_hits < stop_after
            DrawPoint(wid, hgt, dx, dy, max_r, hit_r, cut_r, hits)
            DrawPoint(wid, hgt, dx, dy, max_g, hit_g, cut_g, hits)
            DrawPoint(wid, hgt, dx, dy, max_b, hit_b, cut_b, hits)

            If hits >= draw_every Then
                total_hits += hits
                hits = 0
                DisplayBrot(wid, hgt, max_r, max_g, max_b, hit_r, hit_g, hit_b)

                stop_time = Now
                elapsed = stop_time.Subtract(start_time)
                Me.Text = elapsed.TotalSeconds.ToString("0.00") & " sec, " & _
                    total_hits.ToString & " hits"

                Application.DoEvents()
                If Not m_Drawing Then Exit Do
            End If
        Loop
    End Sub

    ' Plot one point.
    Private Sub DrawPoint(ByVal wid As Integer, ByVal hgt As Integer, ByVal dx As Double, ByVal dy As Double, ByRef max_hits As Integer, ByVal hits(,) As Integer, ByVal cutoff As Integer, ByRef num_hits As Integer)
        Const ESCAPING As Double = 4D

        ' Pick C.
        Dim cx As Double = Wxmin + m_Rand.NextDouble() * (Wxmax - Wxmin)
        Dim cy As Double = Wymin + m_Rand.NextDouble() * (Wymax - Wymin)

        ' Zet Z0.
        Dim x, xx, y, yy As Double
        x = cx
        y = cy
        xx = x * x
        yy = y * y

        ' Iterate.
        For i As Integer = 1 To cutoff
            y = 2 * x * y + cy
            x = xx - yy + cx
            xx = x * x
            yy = y * y
            If xx + yy >= ESCAPING Then Exit For
        Next i

        ' See if we escaped.
        If xx + yy < ESCAPING Then
            ' Plot.
            x = cx
            y = cy
            xx = x * x
            yy = y * y

            ' Iterate.
            For i As Integer = 1 To cutoff
                Dim ix As Integer = CInt((x - Wxmin) / dx)
                Dim iy As Integer = CInt((y - Wymin) / dy)
                If (ix >= 0) AndAlso (ix < hgt) AndAlso (iy >= 0) AndAlso (iy < wid) Then
                    hits(iy, ix) += 1
                    If max_hits < hits(iy, ix) Then max_hits = hits(iy, ix)
                Else
                    Exit For
                End If

                y = 2 * x * y + cy
                x = xx - yy + cx
                xx = x * x
                yy = y * y
                If xx + yy >= ESCAPING Then Exit For
            Next i

            num_hits += 1
        End If
    End Sub

    ' Draw the current image.
    Private Sub DisplayBrot(ByVal wid As Integer, ByVal hgt As Integer, ByVal max_r As Integer, ByVal max_g As Integer, ByVal max_b As Integer, ByVal hit_r(,) As Integer, ByVal hit_g(,) As Integer, ByVal hit_b(,) As Integer)
        Dim gr As Graphics = Graphics.FromImage(m_Bitmap)
        gr.Clear(Color.Black)

        Dim scale_r As Double = 255 * 2.5 / max_r
        Dim scale_g As Double = 255 * 2.5 / max_g
        Dim scale_b As Double = 255 * 2.5 / max_b

        For y As Integer = 0 To hgt - 1
            For x As Integer = 0 To wid - 1
                Dim r As Integer = CInt(hit_r(x, y) * scale_r)
                If r > 255 Then r = 255
                Dim g As Integer = CInt(hit_g(x, y) * scale_g)
                If g > 255 Then g = 255
                Dim b As Integer = CInt(hit_b(x, y) * scale_b)
                If b > 255 Then b = 255

                m_Bitmap.SetPixel(x, y, Color.FromArgb(255, r, g, b))
            Next x
        Next y

        picCanvas.Refresh()
    End Sub
End Class
